@@ -60,7 +60,7 @@ conditional_smooths.brmsfit <- function(x, smooths = NULL,
   subset <- subset_samples(x, subset, nsamples)
   # call as.matrix only once to save time and memory
   samples <- as.matrix(x, subset = subset)
-  bterms <- parse_bf(exclude_terms(x$formula, smooths_only = TRUE))
+  bterms <- brmsterms(exclude_terms(x$formula, smooths_only = TRUE))
   out <- conditional_smooths(
     bterms, fit = x, samples = samples, smooths = smooths, 
     conditions = conditions, int_conditions = int_conditions, 
@@ -157,7 +157,11 @@ conditional_smooths.btl <- function(x, fit, samples, smooths,
     }
     for (cv in byvars) {
       if (cv %in% names(int_conditions)) {
-        values[[cv]] <- int_conditions[[cv]]
+        int_cond <- int_conditions[[cv]]
+        if (is.function(int_cond)) {
+          int_cond <- int_cond(mf[[cv]])
+        }
+        values[[cv]] <- int_cond
       } else if (is.numeric(mf[[cv]])) {
         mean2 <- mean(mf[[cv]], na.rm = TRUE)
         sd2 <- sd(mf[[cv]], na.rm = TRUE)
@@ -179,20 +183,27 @@ conditional_smooths.btl <- function(x, fit, samples, smooths,
       newdata <- newdata[!ex_too_far, ]  
     }
     other_vars <- setdiff(names(conditions), vars)
-    newdata[, other_vars] <- conditions[1, other_vars]
+    for (v in other_vars) {
+      cval <- conditions[1, v]
+      if (length(dim(cval)) == 2L) {
+        # matrix columns don't have automatic broadcasting apparently
+        cval <- matrix(cval, nrow(newdata), ncol(cval), byrow = TRUE)
+      }
+      newdata[[v]] <- cval
+    }
     sdata <- standata(
       fit, newdata, re_formula = NA, 
       internal = TRUE, check_response = FALSE
     )
-    draws_args <- nlist(x, samples, sdata, data = mf)
-    draws <- do_call(extract_draws, draws_args)
+    prep_args <- nlist(x, samples, sdata, data = mf)
+    prep <- do_call(prepare_predictions, prep_args)
     J <- which(smef$termnum == i)
-    scs <- unlist(attr(draws$sm$fe$Xs, "smcols")[J])
-    draws$sm$fe$Xs <- draws$sm$fe$Xs[, scs, drop = FALSE]
-    draws$sm$fe$bs <- draws$sm$fe$bs[, scs, drop = FALSE]
-    draws$sm$re <- draws$sm$re[J]
-    draws$family <- brmsfamily("gaussian")
-    eta <- predictor(draws, i = NULL)
+    scs <- unlist(attr(prep$sm$fe$Xs, "smcols")[J])
+    prep$sm$fe$Xs <- prep$sm$fe$Xs[, scs, drop = FALSE]
+    prep$sm$fe$bs <- prep$sm$fe$bs[, scs, drop = FALSE]
+    prep$sm$re <- prep$sm$re[J]
+    prep$family <- brmsfamily("gaussian")
+    eta <- predictor(prep, i = NULL)
     effects <- na.omit(sub_smef$covars[[1]][1:2])
     marg_data <- add_effects__(newdata[, vars, drop = FALSE], effects)
     if (length(byvars)) {
