@@ -1,12 +1,33 @@
-# rename parameters (and possibly change their dimensions) 
-# within the stanfit object to ensure reasonable parameter names
-# @param x a brmsfit obejct
-# @return a brmfit object with adjusted parameter names and dimensions
+#' Rename Parameters
+#' 
+#' Rename parameters within the \code{stanfit} object after model fitting to
+#' ensure reasonable parameter names. This function is usally called
+#' automatically by \code{\link{brm}} and users will rarely be required to call
+#' it themselves.
+#' 
+#' @param x A brmsfit object.
+#' @return A brmfit object with adjusted parameter names.
+#' 
+#' @examples
+#' \dontrun{
+#' # fit a model manually via rstan
+#' scode <- make_stancode(count ~ Trt, data = epilepsy)
+#' sdata <- make_standata(count ~ Trt, data = epilepsy)
+#' stanfit <- rstan::stan(model_code = scode, data = sdata)
+#' 
+#' # feed the Stan model back into brms
+#' fit <- brm(count ~ Trt, data = epilepsy, empty = TRUE)
+#' fit$fit <- stanfit
+#' fit <- rename_pars(fit)
+#' summary(fit)
+#' }
+#' 
+#' @export
 rename_pars <- function(x) {
   if (!length(x$fit@sim)) {
     return(x) 
   }
-  bterms <- parse_bf(x$formula)
+  bterms <- brmsterms(x$formula)
   data <- model.frame(x)
   meef <- tidy_meef(bterms, data)
   pars <- parnames(x)
@@ -485,7 +506,7 @@ do_renaming <- function(x, change) {
 reorder_pars <- function(x) {
   all_classes <- unique(c(
     "b", "bs", "bsp", "bcs", "ar", "ma", "lagsar", "errorsar", 
-    "car", "sdcar", "sigmaLL", "sd", "cor", "df", "sds", "sdgp", 
+    "car", "sdcar", "cosy", "sd", "cor", "df", "sds", "sdgp", 
     "lscale", valid_dpars(x), "Intercept", "tmp", "rescor", 
     "delta", "lasso", "simo", "r", "s", "zgp", "rcar", "sbhaz", 
     "Ymi", "Yl", "meanme", "sdme", "corme", "Xme", "prior", "lp"
@@ -550,7 +571,7 @@ compute_xi.brmsfit <- function(x, ...) {
 }
 
 #' @export
-compute_xi.mvbrmsdraws <- function(x, fit, ...) {
+compute_xi.mvbrmsprep <- function(x, fit, ...) {
   stopifnot(is.brmsfit(fit))
   for (resp in names(x$resps)) {
     fit <- compute_xi(x$resps[[resp]], fit = fit, ...)
@@ -559,7 +580,7 @@ compute_xi.mvbrmsdraws <- function(x, fit, ...) {
 }
 
 #' @export
-compute_xi.brmsdraws <- function(x, fit, ...) {
+compute_xi.brmsprep <- function(x, fit, ...) {
   stopifnot(is.brmsfit(fit))
   resp <- usc(x$resp)
   tmp_xi_name <- paste0("tmp_xi", resp)
